"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ix_1 = require("@reactive-js/ix");
const rx_1 = require("@reactive-js/rx");
const event_1 = require("./event");
const pipe_1 = require("@reactive-js/pipe");
const disposable_1 = require("@reactive-js/disposable");
const emptyLocation = {
    fragment: "",
    path: "",
    query: "",
};
const locationEquals = (a, b) => a === b ||
    (a.path === b.path && a.query === b.query && a.fragment === b.fragment);
const chopLeadingChar = (str) => str.length > 0 ? str.substring(1) : "";
const getCurrentLocation = (_) => {
    const path = window.location.pathname;
    const query = chopLeadingChar(window.location.search);
    const fragment = chopLeadingChar(window.location.hash);
    return { path, query, fragment };
};
class HistoryIteratorResourceImpl {
    constructor(disposable) {
        this.disposable = disposable;
        this.add = disposable_1.disposableMixin.add;
        this.dispose = disposable_1.disposableMixin.dispose;
        this.remove = disposable_1.disposableMixin.remove;
    }
    get isDisposed() {
        return this.disposable.isDisposed;
    }
    get subscriberCount() {
        return this.disposable.subscriberCount;
    }
    dispatch(newLocation) {
        const currentLocation = getCurrentLocation();
        if (!locationEquals(currentLocation, newLocation)) {
            const { path, query, fragment } = newLocation;
            let uriString = path;
            uriString = query.length > 0 ? `${uriString}?${query}` : uriString;
            uriString = fragment.length > 0 ? `${uriString}#${fragment}` : uriString;
            window.history.pushState(undefined, "", uriString);
        }
    }
    subscribe(subscriber) {
        this.disposable.subscribe(subscriber);
    }
}
const historyIterable = {
    getIXAsyncIterator(scheduler, replayCount) {
        const observable = pipe_1.pipe(rx_1.concat(rx_1.ofValue(getCurrentLocation()), event_1.fromEvent(window, "popstate", getCurrentLocation)), rx_1.publish(scheduler, replayCount));
        return new HistoryIteratorResourceImpl(observable);
    },
};
exports.locationAsyncIterable = ix_1.createPersistentStateAsyncIterable(historyIterable, () => emptyLocation, locationEquals);
//# sourceMappingURL=locationStore.js.map