"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/disposable");
const schedulers_1 = require("@reactive-js/schedulers");
const performance = window.performance;
const Date = window.Date;
const setTimeout = window.setTimeout;
const clearTimeout = window.clearTimeout;
const now = typeof performance === "object" && typeof performance.now === "function"
    ? () => performance.now()
    : () => Date.now();
const yieldInterval = 5;
const maxYieldInterval = 300;
class WebScheduler {
    constructor() {
        this.callCallbackAndDispose = (callback, disposable) => {
            this.startTime = this.now;
            callback();
            disposable.dispose();
        };
        this.channel = new MessageChannel();
        this.shouldYield = navigator !== undefined &&
            navigator.scheduling !== undefined &&
            navigator.scheduling.isInputPending !== undefined
            ? () => {
                const currentTime = now();
                const deadline = this.startTime + yieldInterval;
                const maxDeadline = this.startTime + maxYieldInterval;
                const inputPending = navigator.scheduling.isInputPending();
                return ((currentTime >= deadline && inputPending) ||
                    currentTime >= maxDeadline);
            }
            : () => now() >= this.startTime + yieldInterval;
        this.startTime = this.now;
        this.schedule = schedulers_1.schedulerMixin.schedule;
    }
    get now() {
        return now();
    }
    scheduleCallback(callback, delay = 0) {
        return delay >= 4
            ? this.scheduleDelayed(callback, delay)
            : this.scheduleImmediate(callback);
    }
    scheduleImmediate(callback) {
        const disposable = disposable_1.createDisposable();
        this.channel.port1.onmessage = () => {
            if (!disposable.isDisposed) {
                this.startTime = this.now;
                callback();
                disposable.dispose();
            }
        };
        this.channel.port2.postMessage(null);
        return disposable;
    }
    scheduleDelayed(callback, delay = 0) {
        const disposable = disposable_1.createDisposable(() => clearTimeout(timeout));
        const timeout = setTimeout(this.callCallbackAndDispose, delay, callback, disposable);
        return disposable;
    }
}
let schedulerHost = undefined;
let priorityScheduler = undefined;
exports.createSchedulerWithPriority = (priority) => {
    schedulerHost = schedulerHost || new WebScheduler();
    priorityScheduler =
        priorityScheduler || schedulers_1.createPrioritySchedulerResource(schedulerHost);
    return schedulers_1.createSchedulerWithPriority(priorityScheduler, priority);
};
//# sourceMappingURL=scheduler.js.map