import { createPersistentStateAsyncIterable, } from "@reactive-js/ix";
import { concat, ofValue, publish, } from "@reactive-js/rx";
import { fromEvent } from "./event";
import { pipe } from "@reactive-js/pipe";
import { disposableMixin } from "@reactive-js/disposable";
const emptyLocation = {
    fragment: "",
    path: "",
    query: "",
};
const locationEquals = (a, b) => a === b ||
    (a.path === b.path && a.query === b.query && a.fragment === b.fragment);
const chopLeadingChar = (str) => str.length > 0 ? str.substring(1) : "";
const getCurrentLocation = (_) => {
    const path = window.location.pathname;
    const query = chopLeadingChar(window.location.search);
    const fragment = chopLeadingChar(window.location.hash);
    return { path, query, fragment };
};
class HistoryIteratorResourceImpl {
    constructor(disposable) {
        this.disposable = disposable;
        this.add = disposableMixin.add;
        this.dispose = disposableMixin.dispose;
        this.remove = disposableMixin.remove;
    }
    get isDisposed() {
        return this.disposable.isDisposed;
    }
    get subscriberCount() {
        return this.disposable.subscriberCount;
    }
    dispatch(newLocation) {
        const currentLocation = getCurrentLocation();
        if (!locationEquals(currentLocation, newLocation)) {
            const { path, query, fragment } = newLocation;
            let uriString = path;
            uriString = query.length > 0 ? `${uriString}?${query}` : uriString;
            uriString = fragment.length > 0 ? `${uriString}#${fragment}` : uriString;
            window.history.pushState(undefined, "", uriString);
        }
    }
    subscribe(subscriber) {
        this.disposable.subscribe(subscriber);
    }
}
const historyIterable = {
    getIXAsyncIterator(scheduler, replayCount) {
        const observable = pipe(concat(ofValue(getCurrentLocation()), fromEvent(window, "popstate", getCurrentLocation)), publish(scheduler, replayCount));
        return new HistoryIteratorResourceImpl(observable);
    },
};
export const locationAsyncIterable = createPersistentStateAsyncIterable(historyIterable, () => emptyLocation, locationEquals);
//# sourceMappingURL=locationStore.js.map